/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) WebSec Corporation
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only. Not withstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 *****************************************************************************/
/**
 * @file ConfigAbs.cpp
 * Implementation file for the libconfig abstract interface class that
 * implements methods for reading and writing configuration information
 * from a configuration source.
 */


/*********************************************************************
 * The abstract class that allows instantiation of the
 * OS Dependent implementation classes for determining
 * configuration file information.
 *
 * @author  Dave Webster
 * 
 ********************************************************************/

//////////////////////////////////////////////////////////////////////
// Includes
//////////////////////////////////////////////////////////////////////

#include "ConfigAbs.h"
#include "ConfigImp.h"
#include "ConfigFileImp.h"
#include "ConfigXMLImp.h"
#ifdef WIN32
#include "ConfigRegImp.h"
#endif
#include "StringUtils.h"



///////////////////////////////////////////////////////////////////////////////
// namespace
///////////////////////////////////////////////////////////////////////////////

using namespace std;


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//
//	The constructors must instantiate the correct version of the
//	implementation class depending on the nature of the configuration
//  file while presenting only a single interface to the user. Thus the 
//  abstract constructor must abstract away source specificity. The 
//  choice of the implementation class is based on which number the
//  constructor is invoked with. It defaults to expecting a file
//  implementation for configuration, this is also the implementation
//  if a 0 is passed in. If a 1 is passed to the constructor the
//  configuration source is presumed to be registry information.
//	Lastly, while not currently implemented, a 2 passed into the
//	constructor expects an XML file as the configuration source. These
//	parameters invoke the ConfigFileImp class, the ConfigRegImp, and
//  the ConfigXMLImp classes respectively. Additionally, the constructor
//	accounts for OS specificity by invoking a file configuration
//	implementation if the OS is determined to be UNIX or some other non-
//	WIN32 base OS.
//
//////////////////////////////////////////////////////////////////////

/**
 * Constructor
 */
CConfigAbs::CConfigAbs() 
	: m_pConfigImp(new CConfigFileImp)
{
}


/**
 * Destructor
 */
CConfigAbs::~CConfigAbs()
{

}


/**
 * Constructor
 */
CConfigAbs::CConfigAbs (CConfigAbs::EConfigTypes enumConfigType )
{
	switch ( enumConfigType )
	{
		case CConfigAbs::CFGTYPE_FILE:
			m_pConfigImp.reset(new CConfigFileImp);
			break;
		case CConfigAbs::CFGTYPE_REGISTRY:
#ifdef WIN32
			m_pConfigImp.reset(new CConfigRegImp);
#else //    protect again stupidity for Unix people tring to access the registry
			m_pConfigImp.reset(new CConfigFileImp);
#endif
			break;
		case CConfigAbs::CFGTYPE_XML:
			m_pConfigImp.reset(new CConfigXMLImp);
			break;
		default:
			m_pConfigImp.reset(new CConfigFileImp);
			break;
	}
}


/**
 * Retrieve the list of keys associated with a section. Or in
 * the case of a registry read, all the value names under a
 * subkey
 *
 * @param strSection - The section or subkey string name
 *  
 * @return A list of the key or value strings
 */
list<wstring> CConfigAbs::GetKeysW ( const wstring& strSection )
{
	return m_pConfigImp->GetKeys ( strSection );
}


/**
 *
 * Retrieve the list of keys associated with a section. Or in
 * the case of a registry read, all the value names under a
 * subkey
 * 
 * @param strSection - The section or subkey string name
 * 
 * @return A list of the key or value strings
 */
list<string> CConfigAbs::GetKeysA ( const string& strSection )
{
	wstring wstrSection;
	list<string>   lSections;
	list<wstring> wlSections;
	list<wstring>::iterator it;

	// Make sure the string is a wide string.
	CStringUtils::Widen(wstrSection, strSection);

	// Get the list of keys/values from the specified section.
	wlSections = m_pConfigImp->GetKeys ( wstrSection );

	// Iterate through all keys/values retrieved from the section.
	for (it = wlSections.begin(); it != wlSections.end(); ++it)
	{
		string str;

		// Convert the string back to a narrow string.
		CStringUtils::Narrow(str,*it);
		
		// Add the key/value to the list
		lSections.insert(lSections.end(),str);
	}

	// Return the list of keys/values.
	return lSections;
}


/**
 * Parse the configuration source and store the sections/subkeys
 * and key-value/value name-value data pairs into a hash table 
 * map.
 *
 * @param strFilename - The file name string or the root key string to parse
 * 
 * @return 0 for success or 1 for error
 * 
 */
unsigned long CConfigAbs::ReadConfigW ( const wstring& strFilename )
{
	return m_pConfigImp->ReadConfig ( strFilename );
}


/** 
 * Parse the configuration source and store the sections/subkeys
 * and key-value/value name-value data pairs into a hash table 
 * map.
 * 
 * @param strFilename - The file name string or the root key string to parse
 * 
 * @return 0 for success or 1 for error.
 */
unsigned long CConfigAbs::ReadConfigA ( const string& strFilename )
{
	wstring wstrFilename;
	CStringUtils::Widen(wstrFilename, strFilename);

	return m_pConfigImp->ReadConfig ( wstrFilename );
}



/**
 * Given a section and a key return its value or given a subkey
 * and a value name return its value data.
 *
 * @param strSection The section string or subkey string
 * @param the key to look up in the section of the configuration
 *
 * @return:	The wide string value attached to the input key/value name
 */
wstring CConfigAbs::GetValueW ( const wstring& strSection, const wstring& strKey )
{
	return m_pConfigImp->GetValue ( strSection, strKey );
}


/**
 * Given a section and a key return its value or given a subkey
 * and a value name return its value data.
 *
 * @param strSection The section string or subkey string
 * @param the key to look up in the section of the configuration
 *
 * @return:	The wide string value attached to the input key/value name
 */
string CConfigAbs::GetValueA ( const string& strSection, const string& strKey )
{
	wstring wstrSection, wstrKey;
	string strReturn;

	CStringUtils::Widen(wstrSection, strSection);
	CStringUtils::Widen(wstrKey,strKey);
	CStringUtils::Narrow(strReturn, m_pConfigImp->GetValue ( wstrSection, wstrKey ));

	return strReturn;
}


/**
 * Retrieve all the sections or subkeys in the hash table
 * 
 * @return:	A list of section or subkey strings.
 * 
 */
list<wstring> CConfigAbs::GetSectionNamesW()
{
	return m_pConfigImp->GetSectionNames();
}


/**
 * Retrieve all the sections or subkeys in the hash table
 * 
 * @return:	A list of section or subkey strings.
 * 
 */
list<string> CConfigAbs::GetSectionNamesA()
{
	string wstrSection;
	list<string>   lSections;
	list<wstring> wlSections;
	list<wstring>::iterator it;


	wlSections =  m_pConfigImp->GetSectionNames();

	for (it = wlSections.begin(); it != wlSections.end(); ++it)
	{
		string str;
		CStringUtils::Narrow(str, *it);
		
		lSections.insert(lSections.end(), str);
	}

	return lSections;
}


/**
 * Save the information in the table out to a file
 * 
 * @param strFilename - A string containing the file name to save the file.
 *
 * @return 0 for success or 1 for failure
 * @notes This function has only real significance to config file parsing.
 *        It saves the configuration file output after normalization
 *        to another file.
 */
unsigned long CConfigAbs::SaveConfigW ( const wstring& strFilename )
{
	return m_pConfigImp->SaveConfig ( strFilename );
}


/**
 * Save the information in the table out to a file
 * 
 * @param strFilename - A string containing the file name to save the file.
 *
 * @return 0 for success or 1 for failure
 * @notes This function has only real significance to config file parsing.
 *        It saves the configuration file output after normalization
 *        to another file.
 */
unsigned long CConfigAbs::SaveConfigA ( const string& strFilename )
{
	wstring wstrFilename;

	CStringUtils::Widen(wstrFilename, strFilename);

	return m_pConfigImp->SaveConfig ( wstrFilename );
}


/**
 * Save the information in the table out to the same file read in.
 *
 * @return:	0 for success and >0 for failure
 *
 */
unsigned long CConfigAbs::SaveConfig( )
{
	return m_pConfigImp->SaveConfig ( );
}

	
/**
 * Given a section and a key set its value or given a subkey
 * and a value name set its value data.
 * 
 * @param strSection - The section string or subkey string, the key string or
 *                     value name string
 * @pamam strValue - the value string or value data string to be set.
 * 
 * @return True if the value was set, else false.
 */
bool CConfigAbs::SetValueW ( const wstring& strSection, const wstring& strKey, const wstring& strValue)
{
	return m_pConfigImp->SetValue ( strSection, strKey, strValue );
}


/**
 * Given a section and a key set its value or given a subkey
 * and a value name set its value data.
 * 
 * @param strSection - The section string or subkey string, the key string or
 *                     value name string
 * @pamam strValue - the value string or value data string to be set.
 * 
 * @return True if the value was set, else false.
 */
bool CConfigAbs::SetValueA ( const string& strSection, const string& strKey, const string& strValue)
{
	wstring wstrSection;
	wstring wstrKey;
	wstring wstrValue;

	// convert the strings
	CStringUtils::Widen(wstrSection, strSection);
	CStringUtils::Widen(wstrKey,strKey);
	CStringUtils::Widen(wstrValue,strValue);

	return m_pConfigImp->SetValue ( wstrSection, wstrKey, wstrValue );
}


/**
 * Determine if a key or value name exists in the hash map table
 * 
 * @param strSection = The section or the subkey to search for.
 * @param strKey - The value name of the strings to search for.
 * 
 * @return	true if it exists or false if it does not exist.
 */
bool CConfigAbs::ExistKeyW ( const wstring& strSection, const wstring& strKey )
{
	return m_pConfigImp->ExistKey ( strSection, strKey );
}


/**
 * Determine if a key or value name exists in the hash map table
 * 
 * @param strSection = The section or the subkey to search for.
 * @param strKey - The value name of the strings to search for.
 * 
 * @return	true if it exists or false if it does not exist.
 */
bool CConfigAbs::ExistKeyA ( const string& strSection, const string& strKey )
{
	wstring wstrSection;
	wstring wstrKey;

	CStringUtils::Widen(wstrSection,strSection);
	CStringUtils::Widen(wstrKey,strKey);

	return m_pConfigImp->ExistKey ( wstrSection, wstrKey );
}


/**
 * Determine if a section or subkey exists in the hash map table
 * 
 * @param strSection - The section or the subkey strings to search for.
 * 
 * @return true if it exists or false if it does not exist.
 */
bool CConfigAbs::ExistSectionW ( const wstring& strSection )
{
	return m_pConfigImp->ExistSection ( strSection );
}


/**
 * Determine if a section or subkey exists in the hash map table
 * 
 * @param strSection - The section or the subkey strings to search for.
 * 
 * @return true if it exists or false if it does not exist.
 */
bool CConfigAbs::ExistSectionA ( const string& strSection )
{
	wstring wstrSection;

	CStringUtils::Widen(wstrSection,strSection);

	return m_pConfigImp->ExistSection ( wstrSection );
}
