/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) ProLogic, Incorporated 2008, 1000 Technology Drive, Fairmont, WV 26554
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only. Not withstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 * $Id: $
 *****************************************************************************/
/**
 * @file ConfigAbs.h
 * Header file for the libconfig abstract interface class that defines
 * methods for reading and writing configuration information from a
 * configuration source.
 */


//////////////////////////////////////////////////////////////////////
// pragmas
//////////////////////////////////////////////////////////////////////

#pragma once


//////////////////////////////////////////////////////////////////////
// includes
//////////////////////////////////////////////////////////////////////

#include <string>
#include <list>


//////////////////////////////////////////////////////////////////////
// typedefs
//////////////////////////////////////////////////////////////////////

#if defined UNICODE || defined _UNICODE
typedef std::list<std::wstring> LISTSTR;
#else
typedef std::list<std::string> LISTSTR;
#endif
typedef std::list<std::string> LISTSTRA;
typedef std::list<std::string> LISTSTRW;


//////////////////////////////////////////////////////////////////////
// forward declarations
//////////////////////////////////////////////////////////////////////

class CConfigImp;


/**
 * Provide an abstract user interface to methods that reads
 * read character keys and values from a configuration source.
 * It can read from either NT initializtion file, NT registry
 * or UNIX initializtion file.
 *
 * There are both wide and standard character interfaces in
 * this class.  All methods can be called without the W
 * or the A at the end. This is the default interface.
 * However, if _UNICODE or UNICODE is defined then the
 * default interface will be the the wide character interface.
 * The one with the W at the end.  If not defined then
 * it will choose the interface with the A at the end.
 * For example:
 *   #define UNICODE
 *   ...
 *   ExistKey(strSection, strKey)
 * will call the ExistKeyW method.
 *
 * Usage: Called by other processes to gather run-time data.
 *																	
 * @author David Webster
 *
 * @todo
 *      Add NT registry writting
 *      Make the cross compile
 *      Add XML file reading and writting
 *      Add RemoveSection function		
 *      Add RemoveKey function			
 *
 * @notes Uses a pointer to reference the concrete methods defined
 *        in the Template Method portion of this implementation.
 */
class CConfigAbs
{
public:

	/**
	 * This enumeration defines the configuration types that can be specified
	 * as the parameter to the @a ConfigAbs parameterized constructor
	 */
	enum EConfigTypes
	{
		CFGTYPE_FILE     = 0,	///< Configuration in file (like .ini)
		CFGTYPE_REGISTRY = 1,	///< Configuration in registry
		CFGTYPE_XML      = 2	///< Configuration in XML
		
	};

	/**
	 * Determine if a key or value name exists in the hash map table
	 *
	 * @param strSection The section that the key should be found in
	 * @param strKey The value name to search for.
	 *
	 * @return true if it exists false if it does not exist
	 */
	bool ExistKeyW ( const std::wstring& strSection, const std::wstring& strKey );


	/**
	 * Determine if a key or value name exists in the hash map table
	 *
	 * @param strSection The section that the key should be found in
	 * @param strKey The value name to search for.
	 *
	 * @return true if it exists false if it does not exist
	 */
	bool ExistKeyA ( const std::string& strSection, const std::string& strKey );


	/**
	 * Determine if a section or subkey exists in the hash map table
	 *
	 * @param The section or the subkey strings to search for.
	 *
	 * @return true if it exists false if it does not exist
	 */
	bool ExistSectionW ( const std::wstring& strSection );

	
	/**
	 * Determine if a section or subkey exists in the hash map table
	 *
	 * @param The section or the subkey strings to search for.
	 *
	 * @return true if it exists false if it does not exist
	 */
	bool ExistSectionA ( const std::string&  strSection );


	/**
	 * Retrieve the std::list of keys associated with a section. Or in
	 * the case of a registry read, all the value names under a
	 * subkey
	 *
	 * @param strSection The section or subkey wide std::string name
	 *
	 * @return	A wide std::string std::list of the keys for a given section.
	 */
	std::list<std::wstring> GetKeysW ( const std::wstring& strSection );


	/**
	 * Retrieve the std::list of keys associated with a section. Or in
	 * the case of a registry read, all the value names under a
	 * subkey
	 *
	 * @param strSection The section or subkey wide std::string name
	 *
	 * @return	A std::string std::list of the keys for a given section.
	 */
	std::list< std::string> GetKeysA ( const  std::string& strSection );


	/**
	 * Retrieve all the sections or subkeys in the hash table.
	 *
	 * @return	A wide std::string std::list of section or subkey strings.
	 */
	std::list<std::wstring> GetSectionNamesW();

	
	/**
	 *Retrieve all the sections or subkeys in the hash table.
	 *
	 * @return	A std::list of section or subkey strings.
	 */
	std::list< std::string> GetSectionNamesA();


	/**
	 * Given a section and a key return its value or given a subkey
	 * and a value name return its value data.
	 *
	 * @param strSection The section std::string or subkey std::string
	 * @param the key to look up in the section of the configuration
	 *
	 * @return:	The wide std::string value attached to the input key/value name
	 */
	std::wstring GetValueW ( const std::wstring& strSection, const std::wstring& strKey );


	/**
	 * Given a section and a key return its value or given a subkey
	 * and a value name return its value data.
	 *
	 * @param strSection The section std::string or subkey std::string
	 * @param the key to look up in the section of the configuration
	 *
	 * @return:	The std::string value attached to the input key/value name
	 */
	std::string  GetValueA (  const std::string& strSection,  const std::string& strKey );

	
	/**
	 * Parse the configuration source and store the sections/subkeys
	 * and key-value/value name-value data pairs into a hash table
	 * map.
	 *
	 * @param strFilename The file name wide std::string or the root key
	 *                    wide std::string to parse
	 *
	 * @return	0 for success and >0 for error
	 */
	unsigned long ReadConfigW ( const std::wstring& strFilename );


	/**
	 * Parse the configuration source and store the sections/subkeys
	 * and key-value/value name-value data pairs into a hash table
	 * map.
	 *
	 * @param strFilename The file name std::string or the root key
	 *                    std::string to parse
	 *
	 * @return	0 for success and >0 for error
	 */
	unsigned long ReadConfigA ( const std::string&  strFilename );

	

	/**
	 * Given a section and a key set its value or given a subkey
	 * and a value name set its value data.
	 *
	 * @param strSection The section std::string or subkey std::string,
	 * @param strKey The key std::string or value name std::string
	 * @param strValue The value std::string or value data
	 *                 std::string to be set.
	 *
	 * @return 	True if the value was set, else false.
	 */
	bool SetValueW ( const std::wstring& strSection, const std::wstring& strKey, const std::wstring& strValue );


	/**
	 * Given a section and a key set its value or given a subkey
	 * and a value name set its value data.
	 *
	 * @param strSection The section std::string or subkey std::string,
	 * @param strKey The key std::string or value name std::string
	 * @param strValue The value std::string or value data
	 *                 std::string to be set.
	 *
	 * @return 	True if the value was set, else false.
	 */
	bool SetValueA ( const  std::string& strSection, const  std::string& strKey, const  std::string& strValue );


	/**
	 * Save the information in the table out to a file
	 *
	 * @param strFilename A wide std::string containing the file name
	 *                    to save the file.
	 *
	 * @return:	0 for success and >0 for failure
	 *
	 * @notes This function has only real significance to config file parsing.
	 *        It saves the configuration file output after normalization
	 *        to another file.
	 */
	unsigned long SaveConfigW ( const std::wstring& strFilename );


	/**
	 * Save the information in the table out to a file
	 *
	 * @param strFilename A std::string containing the file name to save the file.
	 *
	 * @return:	0 for success and >0 for failure
	 *
	 * @notes This function has only real significance to config file parsing.
	 *        It saves the configuration file output after normalization
	 *        to another file.
	 */
	unsigned long SaveConfigA ( const  std::string& strFilename );


	/**
	 * Save the information in the table out to the same file read in.
	 *
	 * @return:	0 for success and >0 for failure
	 *
	 */
	unsigned long SaveConfig( );


	/**
	 * Constructor
	 *
	 * The constructors must instantiate the correct version of the
	 * implementation class depending on the nature of the configuration
	 * file while presenting only a single interface to the user. Thus the
	 * abstract constructor must abstract away source specificity. The
	 * choice of the implementation class is based on which number the
	 * constructor is invoked with. It defaults to expecting a file
	 * implementation for configuration, this is also the implementation
	 * if a 0 is passed in. If a 1 is passed to the constructor the
	 * configuration source is presumed to be registry information.
	 * Lastly, while not currently implemented, a 2 passed into the
	 * constructor expects an XML file as the configuration source. These
	 * parameters invoke the ConfigFileImp class, the ConfigRegImp, and
	 * the ConfigXMLImp classes respectively. Additionally, the constructor
	 * accounts for OS specificity by invoking a file configuration
	 * implementation if the OS is determined to be UNIX or some other non-
	 * WIN32 base OS.
	 */
	CConfigAbs ( EConfigTypes nCount );


	/**
	 * Constructor
	 *
	 * The constructors must instantiate the correct version of the
	 * implementation class depending on the nature of the configuration
	 * file while presenting only a single interface to the user. Thus the
	 * abstract constructor must abstract away source specificity. The
	 * choice of the implementation class is based on which number the
	 * constructor is invoked with. It defaults to expecting a file
	 * implementation for configuration, this is also the implementation
	 * if a 0 is passed in. If a 1 is passed to the constructor the
	 * configuration source is presumed to be registry information.
	 * Lastly, while not currently implemented, a 2 passed into the
	 * constructor expects an XML file as the configuration source. These
	 * parameters invoke the ConfigFileImp class, the ConfigRegImp, and
	 * the ConfigXMLImp classes respectively. Additionally, the constructor
	 * accounts for OS specificity by invoking a file configuration
	 * implementation if the OS is determined to be UNIX or some other non-
	 * WIN32 base OS.
	 */
	CConfigAbs();


	/**
	 * Destructor
	 */
	virtual ~CConfigAbs();

private:
	/// member to hold the implementation
	std::auto_ptr<CConfigImp> m_pConfigImp;
	
	/**
	* Copy Constructor declared private so the object cannot be copied.
	*/
	CConfigAbs( const CConfigAbs& );

	/**
	* Assinment operator declared private so the object cannot be copied.
	*/
	CConfigAbs& operator=( const CConfigAbs& );

	/**
	* Equality operator declared private so the object cannot be compared.
	*/
	bool operator==( const CConfigAbs& ) const;
};


///////////////////////////////////////////////////////////////////////////////
// defines (UNICODE/MBCS)
///////////////////////////////////////////////////////////////////////////////
#if defined (_UNICODE) || defined (UNICODE)
#define	ExistKey ExistKeyW
#define ExistSection ExistSectionW
#define GetKeys GetKeysW
#define GetSectionNames GetSectionNamesW
#define GetValue GetValueW
#define ReadConfig ReadConfigW
#define SetValue SetValueW
#else
#define	ExistKey ExistKeyA
#define ExistSection ExistSectionA
#define GetKeys GetKeysA
#define GetSectionNames GetSectionNamesA
#define GetValue GetValueA
#define ReadConfig ReadConfigA
#define SetValue SetValueA
#endif
