/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) WebSec Corporation
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only.   Notwithstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 *****************************************************************************/
/**
 * @file ConfigImp.cpp
 * Implementation file for the libconfig abstract class that implements methods
 * for reading and writing configuration file information.
 */


/**
 * This is the implementation class for returning configuration information
 * @todo:
 *		Improved Error Handling
 *		RemoveKey()
 *		RemoveSection()
 */

//////////////////////////////////////////////////////////////////////
// includes
//////////////////////////////////////////////////////////////////////

#include "ConfigImp.h"
#include <ctype.h>

using namespace std;

typedef pair <wstring, wstring > apair;

/**
 * Constructor
 *
 * These are dummy constructors. Concrete instatiation must be done
 * in the OS dependent implementation classes.
 */
CConfigImp::CConfigImp()
{
	m_szFilename = L"";
}


/**
 * Destructor
 */
CConfigImp::~CConfigImp()
{

	for (m_mapConfigIterator = m_mapConfig.begin();
	     m_mapConfigIterator != m_mapConfig.end();
		 m_mapConfigIterator++)
	{
		delete ((*m_mapConfigIterator).second );
	}
}


/** 
 * Parse the configuration source and store the sections/subkeys
 * and key-value/value name-value data pairs into a hash table 
 * map.
 * 
 * @param strFilename The file name wide string or the root key 
 *                    wide string to parse
 * 
 * @return	0 for success and >0 for error
 */
unsigned long CConfigImp::ReadConfig(wstring filename)
{
	// remove warning
	filename = L"";

	return 0; // status success
}


/**
 * Retrieve the list of keys associated with a section. Or in
 * the case of a registry read, all the value names under a
 * subkey
 * 
 * @param strSection The section or subkey wide string name
 * 
 * @return	A wide string list of the keys for a given section.
 */
list<wstring> CConfigImp::GetKeys(wstring section)
{
	list<wstring> listStr;
	
	// lowercase the section name
	this->LowerCase(&section);

	m_mapConfigIterator = m_mapConfig.find(section); //find the correct section
	if (m_mapConfigIterator != m_mapConfig.end())    // better always be the case
	{
		int iCount;
		KEY2VALUEMAP::iterator iterator;
		wstring str;

		// if nothing is contained then return empty list
		iCount = (*(*m_mapConfigIterator).second).size();
		if (iCount == 0)
			return listStr;
		
		for (iterator = (*(*m_mapConfigIterator).second).begin();
		     iterator != (*(*m_mapConfigIterator).second).end();
			 iterator++)
		{
		    listStr.insert( listStr.end(), (*iterator).first );
		}
	}


	return listStr;
}


/**
 * Given a section and a key return its value or given a subkey
 * and a value name return its value data.
 *
 * @param strSection The section string or subkey string 
 * @param the key to look up in the section of the configuration
 *
 * @return:	The wide string value attached to the input key/value name
 */
wstring CConfigImp::GetValue(wstring section, wstring key)
{
	// lowercase the key and section name
	this->LowerCase(&section);
	this->LowerCase(&key);


	//add key and value to hash table (map)
	m_mapConfigIterator = m_mapConfig.find(section); //find the correct section

	if (m_mapConfigIterator != m_mapConfig.end())    // better always be the case
	{
		KEY2VALUEMAP::iterator myIterator;

		myIterator = (*(*m_mapConfigIterator).second).find(key);
		if (myIterator == (*(*m_mapConfigIterator).second).end())
		{
			// key not found
			return L"";
		}
		else
		{
			return (*myIterator).second;
		}
	}
	
	return L"";
}


/**
 * Take a given string and make it all lower case characters.
 * 
 * @param str - The section string or subkey string and the key string or
 *              value name string.
 * 
 * @return The value string attached to the input key/value name
 */
#ifndef UNDER_CE
void CConfigImp::LowerCase(string *str, const locale* pLoc)
{
	if (pLoc == NULL)
	{
		// lowercase the str
		for (size_t i = 0; i<str->length(); ++i)
		{
			str->at(i) = tolower(str->at(i));
		}
	}
	else
	{
		// lowercase the str
		for (size_t i = 0; i<str->length(); ++i)
		{
			str->at(i) = tolower(str->at(i),*pLoc);
		}
	}

}
#else
void CConfigImp::LowerCase(string *str)
{
	// lowercase the str
	for (size_t i = 0; i<str->length(); ++i)
	{
		str->at(i) = tolower(str->at(i));
	}
}
#endif


/**
 * Take a given string and make it all lower case characters.
 * 
 * @param str - The section string or subkey string and the key string or
 *              value name string.
 * 
 * @return The value string attached to the input key/value name
 */
#ifndef UNDER_CE
void CConfigImp::LowerCase(wstring *str, const locale* pLoc)
{
	if (pLoc == NULL)
	{
		// lowercase the str
		for (size_t i = 0; i<str->length(); ++i)
		{
			str->at(i) = tolower(str->at(i));
		}
	}
	else
	{
		// lowercase the str
		for (size_t i = 0; i<str->length(); ++i)
		{
			str->at(i) = tolower(str->at(i),*pLoc);
		}
	}

}
#else
void CConfigImp::LowerCase(wstring *str)
{
	// lowercase the str
	for (size_t i = 0; i<str->length(); ++i)
	{
		str->at(i) = towlower(str->at(i));
	}
}
#endif

/**
 * Retrieve all the sections or subkeys in the hash table.
 * 
 * @return	A wide string list of section or subkey strings.
 */
list<wstring> CConfigImp::GetSectionNames()
{
	list<wstring> listStr;
	int iCount;

	iCount = m_mapConfig.size();
	if (iCount == 0)
		return listStr;

	for (m_mapConfigIterator = m_mapConfig.begin();
	     m_mapConfigIterator != m_mapConfig.end();
		 m_mapConfigIterator++)
	{
		listStr.insert( listStr.end(), (*m_mapConfigIterator).first );
	}

	return listStr;
}

/**
 * Save the information in the table out to a file
 * 
 * @param strFilename A string containing the file name to save the file.
 * 
 * @return:	0 for success and >0 for failure
 *
 * @notes This function has only real significance to config file parsing.
 *        It saves the configuration file output after normalization
 *        to another file.
 */
unsigned long CConfigImp::SaveConfig(wstring)
{

	return 0;
}


/**
 * Save the information in the table out to the same file read in.
 *
 * @return:	0 for success and >0 for failure
 *
 */
unsigned long CConfigImp::SaveConfig( )
{
	return SaveConfig ( m_szFilename );
}


/**
 * Given a section and a key set its value or given a subkey
 * and a value name set its value data.
 * 
 * @param strSection The section string or subkey string, 
 * @param strKey The key string or value name string
 * @param strValue The value string or value data
 *                 string to be set.
 *
 * @return 	True if the value was set, else false.
 */
bool CConfigImp::SetValue(wstring strSection, wstring strKey, wstring strValue)
{
	bool bResult = false;

	// make lowercase
	LowerCase(&strSection);
	LowerCase(&strKey);
	KEY2VALUEMAP* pMapKeyValue = NULL;

	// we now have a strSection so lets add to the hash table (map)	   
	m_mapConfigIterator = m_mapConfig.find(strSection);

	if (m_mapConfigIterator == m_mapConfig.end())
	{
		// need to add to hash table
		pMapKeyValue = new KEY2VALUEMAP;  // create empty hash table
		m_mapConfig[strSection] = pMapKeyValue;

	}

	// we have made sure the section exists above so find it again
	m_mapConfigIterator = m_mapConfig.find(strSection);

	pMapKeyValue = m_mapConfigIterator->second;

	std::map<std::wstring,std::wstring>::iterator itKeyValue;

	itKeyValue = pMapKeyValue->find(strKey);

	if ( itKeyValue != pMapKeyValue->end() )
	{
		itKeyValue->second = strValue;
	}
	else
	{
		apair p(strKey, strValue);

		pMapKeyValue->insert(p);
	}


	return true;
}

/** 
 * Determine if a key or value name exists in the hash map table
 * 
 * @param strSection The section that the key should be found in
 * @param strKey The value name to search for.
 * 
 * @return true if it exists false if it does not exist
 */
bool CConfigImp::ExistKey(wstring section, wstring key)
{
	// locals
	list<wstring> listsec;
	list<wstring> listkey;
	list<wstring>::iterator itSec;
	list<wstring>::iterator itKey;

	// make section and key lower case for comparison
	this->LowerCase(&section);
	this->LowerCase(&key);

	listsec = this->GetSectionNames();

	for (itSec = listsec.begin(); itSec != listsec.end(); itSec++)
	{
		
		if (*itSec == section)
		{
			listkey = this->GetKeys(*itSec);

			for (itKey = listkey.begin(); itKey != listkey.end(); itKey++)
			{	
				if (*itKey == key)
				{
					return true;
				}
			}
		}
	}

	return false;
}


/** 
 * Determine if a section or subkey exists in the hash map table
 * 
 * @param The section or the subkey strings to search for.
 * 
 * @return true if it exists false if it does not exist
 */
bool CConfigImp::ExistSection(wstring section)
{
	// locals
	list<wstring> listsec;
	list<wstring>::iterator itSec;

	// make section lower case for comparison
	this->LowerCase(&section);

	listsec = this->GetSectionNames();

	for (itSec = listsec.begin(); itSec != listsec.end(); itSec++)
	{
		
		if (*itSec == section)
		{
			return true;
		}
	}

	return false;
}
