/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) ProLogic, Incorporated 2008, 1000 Technology Drive, Fairmont, WV 26554
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only. Not withstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 * $Id: $
 *****************************************************************************/
/**
 * @file
 * Header file for helper methods that convert object from one type
 * to another.
 */


///////////////////////////////////////////////////////////////////////////////
// pragmas
///////////////////////////////////////////////////////////////////////////////

#pragma once


///////////////////////////////////////////////////////////////////////////////
// includes
///////////////////////////////////////////////////////////////////////////////
#include "Config.h"
#include <string>
#include <stdlib.h>


///////////////////////////////////////////////////////////////////////////////
// namespaces
///////////////////////////////////////////////////////////////////////////////

/**
 * This class provides methods that will convert objects from one type to
 * another.
 */
namespace CConvert
{
	/** 
	 * Converts a value into a multibyte string.
	 *
	 * @param nValue A size_t value to be converted.
	 *
	 * @return A std::string based on the input value.
	 */
	std::string IntToStrA(size_t nValue);

	/** 
	 * Converts a value into a multibyte string.
	 *
	 * @param nValue An integer value to be converted.
	 *
	 * @return A std::string based on the input value.
	 */
	std::string IntToStrA(int nValue);

	/** 
	 * Converts an unsigned character into a multibyte string.
	 *
	 * @param cValue A char to be converted to a string.
	 *
	 * @return std::string based on the input value.
	 */
	std::string  UCharToStrA(unsigned char cValue);

	/** 
	 * Converts a character into a multibyte string.
	 *
	 * @param cValue A unsigned char to be converted to a string.
	 *
	 * @return std::string based on the input value.
	 */
	std::string  CharToStrA(char cValue);

	/** 
	 * Swap the byte order for a short value.
	 *
	 * @param nNum A short value to be swapped.
	 *
	 * @return A short value that is the swapped value.
	 */
	short Swap16(short nNum);

	/** 
	 * Swap the byte order for an unsigned short value.
	 *
	 * @param nNum An unsigned short value to be swapped.
	 *
	 * @return An unsigned short value that is the swapped value.
	 */
	unsigned short Swap16(unsigned short nNum);

	/** 
	 * Swap the byte order for a int value.
	 *
	 * @param nNum An integer value to be swapped.
	 *
	 * @return An integer value that is the swapped value.
	 */
	int Swap32(int nNum);

	/** 
	 * Swap the byte order for an unsigned int value.
	 *
	 * @param nNum An unsigned integer value to be swapped.
	 *
	 * @return An unsigned integer value that is the swapped value.
	 */
	unsigned int Swap32(unsigned int nNum);

	/** 
	 * Swap the byte order for a long value.
	 *
	 * @param nNum A long value to be swapped.
	 *
	 * @return A long value that is the swapped value.
	 */
	long Swap32(long nNum);

	/** 
	 * Swap the byte order for an unsigned long value.
	 *
	 * @param nNum An unsigned long value to be swapped.
	 *
	 * @return An unsigned long value that is the swapped value.
	 */
	unsigned long Swap32(unsigned long nNum);

	/**
	 * This will convert a hexidecimal string in readable format to a vector of unsigned characters.
	 *
	 * @param pcIn - unsigned char* that is the pointer to the input data. 
	 * @param nInLen - is the number of characters in the input data to convert.
	 *
	 * @return A hexidecial string representation of the binary data.
	 */
	std::string BinToHexStringA(const unsigned char* pcIn, size_t nInLength);

	/**
	 * This will convert a string that is that is in readable hex format into binary.
	 *
	 * @param str - const string& str that is the input hexidicimal string.
	 * @param pcOut - unsigned char* that is the pointer in which to put the data. Note that this must be allocated
	 *                prior to passing into this function.  The length sould be 1/2 of the string input length.
	 * @param nOutLen - is the number of characters in the output buffer string. Upon return this will set the
	 *                  number of binary digits.
	 *
	 * @note It will throw and error if the string is of 0 length.
	 */
	void HexStringToBin(const std::string& str, unsigned char* pcOut, size_t& nOutLength);

	std::string IntToHexStringA(int nNum);
	std::string IntToHexStringA(unsigned int nNum);

	std::string ShortToHexStringA(short nNum);
	std::string ShortToHexStringA(unsigned short nNum);

	/**
	 * This checks that an integer is divisable by a number.
	 *
	 * @param nLen - int to test if it is devisable by nBase.
	 * @param nBase - the number to use as the devisor.
	 * 
	 * @return True if the integer is a multiple of the base number, otherwise false.
	 */
	bool CheckInputDiv(size_t nLen, size_t nBase);


/*@{
 * Wide strings are not universally available so we wrap them for conditional compilation
 * here.  As far as I can tell there's not a standard test we can use so this is a hack.
 * Feel free to improve upon.  The wish as that we don't have to invent a special pre-
 * processor symbol for our projects but instead base this check on ones already in use.
 *@}
 */
#if defined(HAS_WSTRING)

	/** 
	 * Converts a value into a wide string.
	 *
	 * @param nValue A size_t value to be converted.
	 *
	 * @return A std::wstring based on the input value.
	 */
	std::wstring IntToStrW(size_t nValue);

	/** 
	 * Converts a value into a wide string.
	 *
	 * @param nValue An integer value to be converted.
	 *
	 * @return std::wstring based on the input value.
	 */
	std::wstring IntToStrW(int nValue);

	/** 
	 * Converts an unsigned character into a wide string.
	 *
	 * @param cValue An unsigned char to be converted to a string.
	 *
	 * @return std::wstring based on the input value.
	 */
	std::wstring UCharToStrW(unsigned char cValue);

	/** 
	 * Converts a character into a wide string.
	 *
	 * @param cValue A char to be converted to a string.
	 *
	 * @return std::wstring based on the input value.
	 */
	std::wstring CharToStrW(char cValue);

	/**
	 * This will convert a hexidecimal string in readable format to a vector of unsigned characters.
	 *
	 * @param pcIn - unsigned char* that is the pointer to the input data. 
	 * @param nInLen - is the number of characters in the input data to convert.
	 *
	 * @return A hexidecial string representation of the binary data.
	 */
	std::wstring BinToHexStringW(const unsigned char* pcIn, size_t nInLength);

	/**
	 * This will convert a string that is that is in readable hex format into binary.
	 *
	 * @param str - const string& str that is the input hexidicimal string.
	 * @param pcOut - unsigned char* that is the pointer in which to put the data. Note that this must be allocated
	 *                prior to passing into this function.  The length sould be 1/2 of the string input length.
	 * @param nOutLen - is the number of characters in the string.
	 *
	 * @note It will throw and error if the string is of 0 length.
	 */
	void HexStringToBin(const std::wstring& str, unsigned char* pcOut, size_t& nOutLength);

	std::wstring IntToHexStringW(int nNum);
	std::wstring IntToHexStringW(unsigned int nNum);

	std::wstring ShortToHexStringW(short nNum);
	std::wstring ShortToHexStringW(unsigned short nNum);

#endif //test for wstring
}


