/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) ProLogic, Incorporated 2008, 1000 Technology Drive, Fairmont, WV 26554
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only.   Notwithstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 * $Id: $
 *****************************************************************************/
/**
 * @file SubjectTest.cpp
 * Main header file for the PROJECT_NAME application.
 * 
 * @todo Fix file description.
 */

#include "SubjectTest.h"
#include "Subject.h"

using namespace std;



namespace //private to this file
{
	struct LessProxy : public _CObserverProxy
	{
		const unsigned char *pBegin, *pEnd;

		template<size_t n>
		LessProxy( const unsigned char (&sz)[n] ) : pBegin(sz), pEnd(sz + n) {}

		void operator()() {} //required by parent
		pair<const unsigned char*, const unsigned char*> hash() const
		{
			return pair<const unsigned char*, const unsigned char*>( pBegin, pEnd );
		}
	};
}



void CSubjectTest::TestLess()
{
	const unsigned char sz1[] = { 0 };
	const unsigned char sz2[] = { ~0 };
	const unsigned char sz3[] = { 0, 0 };

	CPPUNIT_ASSERT_EQUAL( false, (LessProxy)sz1 < (LessProxy)sz1 );
	CPPUNIT_ASSERT_EQUAL( true, (LessProxy)sz1 < (LessProxy)sz2 );
	CPPUNIT_ASSERT_EQUAL( false, (LessProxy)sz2 < (LessProxy)sz1 );
	CPPUNIT_ASSERT_EQUAL( true, (LessProxy)sz1 < (LessProxy)sz3 );
	CPPUNIT_ASSERT_EQUAL( false, (LessProxy)sz3 < (LessProxy)sz1 );
}


void CSubjectTest::TestGetSet()
{
	CSubject<int> nSubject(0);

	CPPUNIT_ASSERT_EQUAL( 0, nSubject.Get());
	nSubject.Set( -4 );
	CPPUNIT_ASSERT_EQUAL( -4, nSubject.Get());
	CPPUNIT_ASSERT_EQUAL( 92, nSubject.Set( 92 ));
}


struct CObserver
{
	bool updated;

	CObserver() : updated(false) {}

	void update( const int& ) { updated = true; }
	void dummy( const int& ) { /*do nothing*/ }
	void throws( const int& ) { throw 0; }
};



void CSubjectTest::TestAttach()
{
	CSubject<int> nSubject(0);
	CObserver obs1;
	CObserver obs2;

	// assert expected initial conditions
	CPPUNIT_ASSERT_EQUAL( false, obs1.updated );
	CPPUNIT_ASSERT_EQUAL( false, obs2.updated );

	nSubject.Set( -6 );

	// haven't attached yet so shouldn't've been notified
	CPPUNIT_ASSERT_EQUAL( false, obs1.updated );
	CPPUNIT_ASSERT_EQUAL( false, obs2.updated );

	nSubject.Attach( obs1, &CObserver::update );
	nSubject.Attach( obs2, &CObserver::update );

	// haven't re-updated so still shouldn't've been notified
	CPPUNIT_ASSERT_EQUAL( false, obs1.updated );
	CPPUNIT_ASSERT_EQUAL( false, obs2.updated );

	nSubject.Update();

	// NOW we expect to see something
	CPPUNIT_ASSERT_EQUAL( true, obs1.updated );
	CPPUNIT_ASSERT_EQUAL( true, obs2.updated );
}


void CSubjectTest::TestDetachUnattached()
{
	CSubject<int> nSubject(0);
	CObserver obs1;

	// and let's detach something that's not attached
	// shouldn't do anything, just want to make sure it doesn't error
	nSubject.Detach( obs1, &CObserver::dummy );

	nSubject.Set( -6 );

	// haven't attached yet so shouldn't've been notified
	CPPUNIT_ASSERT_EQUAL( false, obs1.updated );

	nSubject.Attach( obs1, &CObserver::update );

	// haven't re-updated so still shouldn't've been notified
	CPPUNIT_ASSERT_EQUAL( false, obs1.updated );

	nSubject.Update();

	// NOW we expect to see something
	CPPUNIT_ASSERT_EQUAL( true, obs1.updated );

	// reset
	obs1.updated = false;

	// and let's detach something that's not attached, again
	nSubject.Detach( obs1, &CObserver::dummy );

	nSubject.Update();

	// and we should still be getting updates
	CPPUNIT_ASSERT_EQUAL( true, obs1.updated );
}


void CSubjectTest::TestDetach()
{
	CSubject<int> nSubject(0);
	CObserver obs1;
	CObserver obs2;

	nSubject.Attach( obs1, &CObserver::update );
	nSubject.Attach( obs2, &CObserver::update );

	nSubject.Update();

	// expect to see an update
	CPPUNIT_ASSERT_EQUAL( true, obs1.updated );
	CPPUNIT_ASSERT_EQUAL( true, obs2.updated );

	// reset
	obs1.updated = false;
	obs2.updated = false;

	// detach one
	nSubject.Detach( obs1, &CObserver::update );

	nSubject.Update();

	// expect to see one update, not two
	CPPUNIT_ASSERT_EQUAL( false, obs1.updated );
	CPPUNIT_ASSERT_EQUAL( true, obs2.updated );
}


void CSubjectTest::TestThrowDoesntDisrupt()
{
	CSubject<int> nSubject(0);
	CObserver obs1;
	CObserver obs2;

	//observer 1 throws an exception on update
	nSubject.Attach( obs1, &CObserver::throws );
	nSubject.Attach( obs2, &CObserver::update );

	nSubject.Update();

	// expect to still see an update on observer 2
	CPPUNIT_ASSERT_EQUAL( true, obs2.updated );
}


//void tests ---------------------------------------------------------------------------------------------------------
struct CVoidObserver
{
	bool updated;

	CVoidObserver() : updated(false) {}

	void update() { updated = true; }
	void dummy() { /*do nothing*/ }
	void throws() { throw 0; }
};



void CSubjectTest::TestVoidAttach()
{
	CSubject<void> vSubject;
	CVoidObserver obs1;
	CVoidObserver obs2;

	// assert expected initial conditions
	CPPUNIT_ASSERT_EQUAL( false, obs1.updated );
	CPPUNIT_ASSERT_EQUAL( false, obs2.updated );

	vSubject.Update();

	// haven't attached yet so shouldn't've been notified
	CPPUNIT_ASSERT_EQUAL( false, obs1.updated );
	CPPUNIT_ASSERT_EQUAL( false, obs2.updated );

	vSubject.Attach( obs1, &CVoidObserver::update );
	vSubject.Attach( obs2, &CVoidObserver::update );

	// haven't re-updated so still shouldn't've been notified
	CPPUNIT_ASSERT_EQUAL( false, obs1.updated );
	CPPUNIT_ASSERT_EQUAL( false, obs2.updated );

	vSubject.Update();

	// NOW we expect to see something
	CPPUNIT_ASSERT_EQUAL( true, obs1.updated );
	CPPUNIT_ASSERT_EQUAL( true, obs2.updated );
}


void CSubjectTest::TestVoidDetachUnattached()
{
	CSubject<void> vSubject;
	CVoidObserver obs1;

	// and let's detach something that's not attached
	// shouldn't do anything, just want to make sure it doesn't error
	vSubject.Detach( obs1, &CVoidObserver::dummy );

	vSubject.Update();

	// haven't attached yet so shouldn't've been notified
	CPPUNIT_ASSERT_EQUAL( false, obs1.updated );

	vSubject.Attach( obs1, &CVoidObserver::update );

	// haven't re-updated so still shouldn't've been notified
	CPPUNIT_ASSERT_EQUAL( false, obs1.updated );

	vSubject.Update();

	// NOW we expect to see something
	CPPUNIT_ASSERT_EQUAL( true, obs1.updated );

	// reset
	obs1.updated = false;

	// and let's detach something that's not attached, again
	vSubject.Detach( obs1, &CVoidObserver::dummy );

	vSubject.Update();

	// and we should still be getting updates
	CPPUNIT_ASSERT_EQUAL( true, obs1.updated );
}


void CSubjectTest::TestVoidDetach()
{
	CSubject<void> vSubject;
	CVoidObserver obs1;
	CVoidObserver obs2;

	vSubject.Attach( obs1, &CVoidObserver::update );
	vSubject.Attach( obs2, &CVoidObserver::update );

	vSubject.Update();

	// expect to see an update
	CPPUNIT_ASSERT_EQUAL( true, obs1.updated );
	CPPUNIT_ASSERT_EQUAL( true, obs2.updated );

	// reset
	obs1.updated = false;
	obs2.updated = false;

	// detach one
	vSubject.Detach( obs1, &CVoidObserver::update );

	vSubject.Update();

	// expect to see one update, not two
	CPPUNIT_ASSERT_EQUAL( false, obs1.updated );
	CPPUNIT_ASSERT_EQUAL( true, obs2.updated );
}


void CSubjectTest::TestVoidThrowDoesntDisrupt()
{
	CSubject<void> nSubject;
	CVoidObserver obs1;
	CVoidObserver obs2;

	//observer 1 throws an exception on update
	nSubject.Attach( obs1, &CVoidObserver::throws );
	nSubject.Attach( obs2, &CVoidObserver::update );

	nSubject.Update();

	// expect to still see an update on observer 2
	CPPUNIT_ASSERT_EQUAL( true, obs2.updated );
}


//reference tests ----------------------------------------------------------------------------------------------------

void CSubjectTest::TestRefGetSet()
{
	int n = 0;
	CSubject<int&> nSubject(n);

	CPPUNIT_ASSERT_EQUAL( 0, nSubject.Get());
	nSubject.Set( -4 );
	CPPUNIT_ASSERT_EQUAL( -4, nSubject.Get());
	CPPUNIT_ASSERT_EQUAL( 92, nSubject.Set( 92 ));
}


void CSubjectTest::TestConstRefGetSet()
{
	int n = 0;
	CSubject<const int&> nSubject(n);

	CPPUNIT_ASSERT_EQUAL( 0, nSubject.Get());
	nSubject.Set( -4 );
	CPPUNIT_ASSERT_EQUAL( -4, nSubject.Get());
	CPPUNIT_ASSERT_EQUAL( 92, nSubject.Set( 92 ));
}


void CSubjectTest::TestRefSetTarget()
{
	//this test makes sure the Subject class is actually storing a reference and not making a copy
	int n = 0;
	CSubject<int&> nSubject(n);

	CPPUNIT_ASSERT_EQUAL( 0, nSubject.Get());
	n = -4;
	CPPUNIT_ASSERT_EQUAL( -4, nSubject.Get());
}


void CSubjectTest::TestRefAttach()
{
	int n = 0;
	CSubject<int&> nSubject(n);
	CObserver obs1;
	CObserver obs2;

	// assert expected initial conditions
	CPPUNIT_ASSERT_EQUAL( false, obs1.updated );
	CPPUNIT_ASSERT_EQUAL( false, obs2.updated );

	nSubject.Set( -6 );

	// haven't attached yet so shouldn't've been notified
	CPPUNIT_ASSERT_EQUAL( false, obs1.updated );
	CPPUNIT_ASSERT_EQUAL( false, obs2.updated );

	nSubject.Attach( obs1, &CObserver::update );
	nSubject.Attach( obs2, &CObserver::update );

	// haven't re-updated so still shouldn't've been notified
	CPPUNIT_ASSERT_EQUAL( false, obs1.updated );
	CPPUNIT_ASSERT_EQUAL( false, obs2.updated );

	nSubject.Update();

	// NOW we expect to see something
	CPPUNIT_ASSERT_EQUAL( true, obs1.updated );
	CPPUNIT_ASSERT_EQUAL( true, obs2.updated );
}


void CSubjectTest::TestRefDetachUnattached()
{
	int n = 0;
	CSubject<int&> nSubject(n);
	CObserver obs1;

	// and let's detach something that's not attached
	// shouldn't do anything, just want to make sure it doesn't error
	nSubject.Detach( obs1, &CObserver::dummy );

	nSubject.Set( -6 );

	// haven't attached yet so shouldn't've been notified
	CPPUNIT_ASSERT_EQUAL( false, obs1.updated );

	nSubject.Attach( obs1, &CObserver::update );

	// haven't re-updated so still shouldn't've been notified
	CPPUNIT_ASSERT_EQUAL( false, obs1.updated );

	nSubject.Update();

	// NOW we expect to see something
	CPPUNIT_ASSERT_EQUAL( true, obs1.updated );

	// reset
	obs1.updated = false;

	// and let's detach something that's not attached, again
	nSubject.Detach( obs1, &CObserver::dummy );

	nSubject.Update();

	// and we should still be getting updates
	CPPUNIT_ASSERT_EQUAL( true, obs1.updated );
}


void CSubjectTest::TestRefDetach()
{
	int n = 0;
	CSubject<int&> nSubject(n);
	CObserver obs1;
	CObserver obs2;

	nSubject.Attach( obs1, &CObserver::update );
	nSubject.Attach( obs2, &CObserver::update );

	nSubject.Update();

	// expect to see an update
	CPPUNIT_ASSERT_EQUAL( true, obs1.updated );
	CPPUNIT_ASSERT_EQUAL( true, obs2.updated );

	// reset
	obs1.updated = false;
	obs2.updated = false;

	// detach one
	nSubject.Detach( obs1, &CObserver::update );

	nSubject.Update();

	// expect to see one update, not two
	CPPUNIT_ASSERT_EQUAL( false, obs1.updated );
	CPPUNIT_ASSERT_EQUAL( true, obs2.updated );
}


void CSubjectTest::TestRefThrowDoesntDisrupt()
{
	int n = 0;
	CSubject<int&> nSubject(n);
	CObserver obs1;
	CObserver obs2;

	//observer 1 throws an exception on update
	nSubject.Attach( obs1, &CObserver::throws );
	nSubject.Attach( obs2, &CObserver::update );

	nSubject.Update();

	// expect to still see an update on observer 2
	CPPUNIT_ASSERT_EQUAL( true, obs2.updated );
}



